from os import getenv

from aiogram import Bot, F, Router
from aiogram.filters import Command
from aiogram.fsm.context import FSMContext
from aiogram.fsm.state import State, StatesGroup
from aiogram.types import (
    CallbackQuery,
    InlineKeyboardButton,
    InlineKeyboardMarkup,
    KeyboardButton,
    Message,
    ReplyKeyboardMarkup,
    ReplyKeyboardRemove,
)
from pyquizAPI import QuizClient


async def get_question():
    api = getenv('QUIZ_API')
    client = QuizClient(api)
    client.make_config(limit=1, difficulty='Easy')

    questions = client.get_questions(use_config=True)
    return questions


router = Router()
kb = [
    [KeyboardButton(text='Quiz')],
    [KeyboardButton(text='Tic_tac_toy')],
    [KeyboardButton(text='Quest')],
]


class GameState(StatesGroup):
    game = State()
    quiz = State()
    tic_tac_toy = State()
    quest = State()


markup_kb = ReplyKeyboardMarkup(keyboard=kb, resize_keyboard=True)


@router.message(Command('games'))
async def games(message: Message):
    await message.answer('Select Game:', reply_markup=markup_kb)


@router.message(F.text.lower() == 'exit game')
async def exit_games(message: Message, state: FSMContext):
    await state.clear()
    await message.reply('Game is finished', reply_markup=ReplyKeyboardRemove())


@router.message(F.text.lower() == 'quiz')
async def with_puree(message: Message, state: FSMContext):
    quiz_kb = [
        [KeyboardButton(text='Next question')],
        [KeyboardButton(text='Exit Game')],
    ]
    await state.set_state(GameState.quiz)

    await message.reply('Start Quiz', reply_markup=ReplyKeyboardRemove())
    await message.reply(
        'First question:',
        reply_markup=ReplyKeyboardMarkup(keyboard=quiz_kb, resize_keyboard=True),
    )
    await question(message, state)


@router.message(GameState.quiz, F.text)
async def question(message: Message, state: FSMContext):
    data = await state.get_data()
    question_data = await get_question()
    question = question_data[0].get('question')
    answers = {
        question_data[0]['answers'][k]: question_data[0]['correct_answers'][
            k + '_correct'
        ]
        for k in question_data[0]['answers'].keys()
        if question_data[0]['answers'][k] is not None
    }
    buttons = []

    idx = 1
    text = f'Question:\n{question}\n\nanswers:\n'

    for k, v in answers.items():
        buttons.append([InlineKeyboardButton(text=f'choice {idx}', callback_data=k)])
        if v == 'true':
            data['answer'] = k
        text = text + f'{idx} - {k}\n'
        idx += 1

    data['question'] = question
    await state.update_data(data)

    await message.answer(
        text=text,
        reply_markup=InlineKeyboardMarkup(inline_keyboard=buttons),
    )


@router.callback_query(GameState.quiz)
async def check_answer(callback: CallbackQuery, state: FSMContext, bot: Bot):
    answer = (await state.get_data())['answer']

    await callback.answer()
    if answer == callback.data:
        await callback.message.answer(text='Correct')
    else:
        await callback.message.answer(text='Opps. You miss')
    await bot.edit_message_text(
        chat_id=callback.from_user.id,
        message_id=callback.message.message_id,
        text=callback.message.text,
    )
